@@ -18,7 +18,8 @@
 #' @param method Optimization method passed to \code{optim}.
 #' @param init Initial values for the maximum likelihood optimization.
 #' If not provided, the optimization will start in 0.
-#' @return An object of class \code{cm} containing the parameters of the cure model.
+#' @return An object of class \code{cm} containing the estimated parameters of the cure model.
+#' The appropriate link functions taken on \eqn{\pi} and the \eqn{\theta_i}'s are linear in the covariates corresponding to their respective parameter estimates.
 #' @details If \code{type = "mixture"}, the function fits the model,
 #' \deqn{S(t|z) = \pi(z) + [1 - \pi(z)] S_u(t|z),}
 #' and if \code{type = "nmixture"}, the function fits the model,
@@ -32,20 +33,23 @@
 #' Weibull model:
 #' \deqn{S_u(t) = \exp\left(-\theta_1 t^{\theta_2}\right).}
 #' Log-normal model:
-#' \deqn{S_u(t) = 1 - \Phi\left(\frac{\log(t) - \theta_1}{\theta_2}\right)}
+#' \deqn{S_u(t) = 1 - \Phi\left(\frac{\log(t) - \theta_1}{\theta_2}\right).}
 #' Weibull-exponential mixture model:
 #' \deqn{S_u(t) = \theta_1\exp\left(-\theta_2 t^{\theta_3}\right) + (1 - \theta_1)\exp\left(-\theta_4 t\right).}
 #' Weibull-Weibull mixture model:
 #' \deqn{S_u(t) = \theta_1\exp\left(-\theta_2 t^{\theta_3}\right) + (1 - \theta_1)\exp\left(-\theta_4 t^{\theta_5}\right).}
-#' In the two last mixture models, the link function for the mixture component is controlled by \code{link.mix}.
+#' Generalized modified Weibull distribution:
+#' \deqn{S_u(t) = 1-\left(1 - \exp\left(-\theta_1 t ^ \theta_2 \exp(\theta_3 t)\right)\right) ^ \theta_4.}
+#' In the Weibull-exponential and Weibull-Weibull mixture models, the link function for the mixture component is controlled by \code{link.mix}.
 #' The remaining parameters are modelled using an exponential link function except \eqn{\theta_1} in the log-normal model,
-#' which is modelled using the identity.
+#' which is modelled using the identity. Parameters are not transformed back to the original scale in
+#' the outputted object and related \code{print.cm} and \code{summary.cm} functions
 #' @export
 #' @example inst/fit.cure.model.ex.R
 
 
 fit.cure.model <- function(formula, data, formula.surv = NULL, type = c("mixture", "nmixture"),
-                           dist = c("weibull", "exponential", "lognormal", "weiwei", "weiexp"),
+                           dist = c("weibull", "exponential", "lognormal", "weiwei", "weiexp", "gmw"),
                            link = c("logit", "loglog", "identity", "probit"), bhazard = NULL,
                            covariance = TRUE, link.mix = c("logit", "loglog", "identity", "probit"),
                            control = list(maxit = 10000), method = "Nelder-Mead", init = NULL){
@@ -60,7 +64,8 @@ fit.cure.model <- function(formula, data, formula.surv = NULL, type = c("mixture
                          exponential = 1,
                          lognormal = 2,
                          weiwei = 5,
-                         weiexp = 4)
+                         weiexp = 4,
+                         gmw = 4)
 
   if(is.null(formula.surv)){
     formula.surv <- rep(list(~1), max.formulas)
@@ -74,8 +79,10 @@ fit.cure.model <- function(formula, data, formula.surv = NULL, type = c("mixture
   #Delete missing observations and extract response data
   all.formulas <- c(formula, formula.surv)
   all_vars <- unique(unlist(lapply(all.formulas, all.vars)))
-  # data.c <- data[complete.cases(data[, all_vars]),]
-  data.c <- data
+  data.c <- stats::na.omit(data[, all_vars])
+  cc <- stats::complete.cases(data[, all_vars])
+  data <- data[cc,]
+  #data.c <- data
 
   #Extract survival time and event variable
   eventExpr <- lhs(formula)[[length(lhs(formula))]]
@@ -176,15 +183,15 @@ fit.cure.model <- function(formula, data, formula.surv = NULL, type = c("mixture
   n.param.formula <- sapply(X.all, ncol)
   #Get initial values
   if(is.null(init)){
-    n.param <- sum(n.param.formula)
-    init <- rep(0, n.param)
+    n.param     <- sum(n.param.formula)
+    init        <- rep(0, n.param)
     names(init) <- unlist(lapply(X.all, colnames))
   }
 
-  optim.args <- c(control = list(control), args)
+  optim.args        <- c(control = list(control), args)
   optim.args$method <- method
-  optim.args$fn <- minusloglik
-  optim.args$par <- init
+  optim.args$fn     <- minusloglik
+  optim.args$par    <- init
 
   #Run optimization
   optim.out <- do.call(optim, optim.args)
@@ -214,12 +221,13 @@ fit.cure.model <- function(formula, data, formula.surv = NULL, type = c("mixture
   L <- list(data = data, all.formulas = all.formulas,
             coefs = coefs, dist = dist, link = link,
             type = type, ci = covariance,
-            ML = optim.out$value, covariance = cov,
+            ML = -optim.out$value, covariance = cov,
             df = nrow(data) - length(optim.out$par),
             optim = optim.out, n.param.formula = n.param.formula,
             surv.fun = surv.fun, dens.fun = dens.fun, optim.args = optim.args,
             time = time, event = event, timeVar = timeVar, link.mix = link.mix,
-            excess = excess, cure.type = cure.type, args = args, lm.objects = lm.objects)
+            excess = excess, cure.type = cure.type, args = args, lm.objects = lm.objects,
+            na.action = attr(data.c, "na.action"))
   class(L) <- c("cm", "cuRe")
   L
 }
@@ -232,55 +240,73 @@ get_design <- function(formula, data){
     data.frame()
 }
 
-
-print.cm <- function(fit){
-  type <- switch(fit$type,
+#' @export
+#' @method print cm
+print.cm <- function(x, ...){
+  type <- switch(x$type,
                  mixture = "mixture",
                  nmixture = "non-mixture")
   cat("Model:\n")
   print(paste0("Parametric ", type, " cure model"))
   cat("Family survival / curerate: \n")
-  print(paste0(fit$dist, " / ", fit$link))
-  is.not.null <- sapply(fit$formulas, function(f) !is.null(f))
-  coef.names <- unlist(lapply(fit$formulas[is.not.null], function(x) Reduce(paste, deparse(x))))
+  print(paste0(x$dist, " / ", x$link))
+  is.not.null <- sapply(x$formulas, function(f) !is.null(f))
+  coef.names <- unlist(lapply(x$formulas[is.not.null], function(x) Reduce(paste, deparse(x))))
   cat("\nCoefficients:\n")
-  coefs <- fit$coefs[sapply(fit$coefs, function(coef) length(coef) != 0)]
+  coefs <- x$coefs[sapply(x$coefs, function(coef) length(coef) != 0)]
   names(coefs) <- coef.names
   print(coefs)
 }
 
+replace_names <- function(coef_list){
+  label_names <- c("1" = "pi", "2" = "theta1",
+                   "3" = "theta2", "4" = "theta3",
+                   "5" = "theta4", "6" = "theta5")
 
-summary.cm <- function(fit){
-  se <- sqrt(diag(fit$cov))
-  coefs <- unlist(fit$coefs)
-  tval <- coefs / se
+  names(coef_list) <- label_names[names(coef_list)]
+  coef_list
+}
+
+#' @export
+#' @method summary cm
+summary.cm <- function(object, ...){
+  se <- sqrt(diag(object$cov))
+  coef_list <- object$coefs
+  coef_list <- replace_names(coef_list)
+  coefs <- unlist(coef_list)
+  z <- coefs / se
   TAB1 <- cbind(Estimate = coefs,
                 StdErr = se,
-                t.value = tval,
-                p.value = ifelse(is.na(tval), rep(NA, length(coefs)),
-                                 2 * pt(-abs(tval), df = fit$df)))
+                z.value = z,
+                p.value = ifelse(is.na(z), rep(NA, length(coefs)),
+                                 2 * pnorm(-abs(z))))
+
 
   results <- list(coefs = TAB1)
-  results$type <- fit$type
-  results$link <- fit$link
-  results$ML <- fit$ML
-  formulas <- fit$formulas
-  names(formulas) <- c("gamma", "k1", "k2", "k3")
+  results$type <- object$type
+  results$link <- object$link
+  results$ML <- object$ML
+  formulas <- object$all.formulas
+  names(formulas) <- names(coef_list)
   results$formulas <- formulas[sapply(formulas, function(x) !is.null(x))]
+  if (!is.null(object$na.action))
+    results$na.action <- object$na.action
   class(results) <- "summary.cm"
   results
 }
 
-
-print.summary.cm <- function(x)
+#' @export
+#' @method print summary.cm
+print.summary.cm <- function(x, ...)
 {
   cat("Calls:\n")
   print(x$formulas)
   #    cat("\n")
-  printCoefmat(x$coefs, P.values = TRUE, has.Pvalue = T)
+  stats::printCoefmat(x$coefs, P.values = TRUE, has.Pvalue = T)
+  if (nzchar(mess <- stats::naprint(x$na.action)))
+    cat("  (", mess, ")\n", sep = "")
   cat("\n")
   cat("Type =", x$type, "\n")
   cat("Link =", x$link, "\n")
   cat("LogLik(model) =", x$ML, "\n")
 }
-
